/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import type { HttpStart } from '@kbn/core/public';
import { decodeOrThrow } from '@kbn/io-ts-utils';
import type {
  CreateMetricsExplorerViewResponsePayload,
  FindMetricsExplorerViewResponsePayload,
  GetMetricsExplorerViewResponsePayload,
  UpdateMetricsExplorerViewResponsePayload,
  CreateMetricsExplorerViewAttributesRequestPayload,
  UpdateMetricsExplorerViewAttributesRequestPayload,
} from '../../../common/http_api/latest';
import {
  createMetricsExplorerViewRequestPayloadRT,
  findMetricsExplorerViewResponsePayloadRT,
  getMetricsExplorerViewUrl,
  metricsExplorerViewResponsePayloadRT,
} from '../../../common/http_api/latest';
import {
  DeleteMetricsExplorerViewError,
  FetchMetricsExplorerViewError,
  UpsertMetricsExplorerViewError,
} from '../../../common/metrics_explorer_views';
import type { IMetricsExplorerViewsClient } from './types';

export class MetricsExplorerViewsClient implements IMetricsExplorerViewsClient {
  constructor(private readonly http: HttpStart) {}

  async findMetricsExplorerViews(): Promise<FindMetricsExplorerViewResponsePayload['data']> {
    const response = await this.http.get(getMetricsExplorerViewUrl()).catch((error) => {
      throw new FetchMetricsExplorerViewError(`Failed to fetch metrics explorer views: ${error}`);
    });

    const { data } = decodeOrThrow(
      findMetricsExplorerViewResponsePayloadRT,
      (message: string) =>
        new FetchMetricsExplorerViewError(`Failed to decode metrics explorer views: ${message}"`)
    )(response);

    return data;
  }

  async getMetricsExplorerView(
    metricsExplorerViewId: string
  ): Promise<GetMetricsExplorerViewResponsePayload> {
    const response = await this.http
      .get(getMetricsExplorerViewUrl(metricsExplorerViewId))
      .catch((error) => {
        throw new FetchMetricsExplorerViewError(
          `Failed to fetch metrics explorer view "${metricsExplorerViewId}": ${error}`
        );
      });

    const { data } = decodeOrThrow(
      metricsExplorerViewResponsePayloadRT,
      (message: string) =>
        new FetchMetricsExplorerViewError(
          `Failed to decode metrics explorer view "${metricsExplorerViewId}": ${message}"`
        )
    )(response);

    return data;
  }

  async createMetricsExplorerView(
    metricsExplorerViewAttributes: CreateMetricsExplorerViewAttributesRequestPayload
  ): Promise<CreateMetricsExplorerViewResponsePayload> {
    const response = await this.http
      .post(getMetricsExplorerViewUrl(), {
        body: JSON.stringify(
          createMetricsExplorerViewRequestPayloadRT.encode({
            attributes: metricsExplorerViewAttributes,
          })
        ),
      })
      .catch((error) => {
        throw new UpsertMetricsExplorerViewError(
          `Failed to create new metrics explorer view: ${error.body?.message ?? error.message}`
        );
      });

    const { data } = decodeOrThrow(
      metricsExplorerViewResponsePayloadRT,
      (message: string) =>
        new UpsertMetricsExplorerViewError(
          `Failed to decode newly written metrics explorer view: ${message}"`
        )
    )(response);

    return data;
  }

  async updateMetricsExplorerView(
    metricsExplorerViewId: string,
    metricsExplorerViewAttributes: UpdateMetricsExplorerViewAttributesRequestPayload
  ): Promise<UpdateMetricsExplorerViewResponsePayload> {
    const response = await this.http
      .put(getMetricsExplorerViewUrl(metricsExplorerViewId), {
        body: JSON.stringify(
          createMetricsExplorerViewRequestPayloadRT.encode({
            attributes: metricsExplorerViewAttributes,
          })
        ),
      })
      .catch((error) => {
        throw new UpsertMetricsExplorerViewError(
          `Failed to update metrics explorer view "${metricsExplorerViewId}": ${
            error.body?.message ?? error.message
          }`
        );
      });

    const { data } = decodeOrThrow(
      metricsExplorerViewResponsePayloadRT,
      (message: string) =>
        new UpsertMetricsExplorerViewError(
          `Failed to decode updated metrics explorer view "${metricsExplorerViewId}": ${message}"`
        )
    )(response);

    return data;
  }

  deleteMetricsExplorerView(metricsExplorerViewId: string): Promise<null> {
    return this.http
      .delete(getMetricsExplorerViewUrl(metricsExplorerViewId))
      .then(() => null)
      .catch((error) => {
        throw new DeleteMetricsExplorerViewError(
          `Failed to delete metrics explorer view "${metricsExplorerViewId}": ${error}`
        );
      });
  }
}
